@@ -19,10 +19,8 @@
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(defconst ghc-error-buffer-name "*GHC Errors*")
-
 (defconst ghc-flymake-allowed-file-name-masks
-  '("\\.l?hs$" ghc-flymake-init flymake-simple-cleanup flymake-get-real-file-name))
+  '("\\.l?hs$" ghc-flymake-init flymake-simple-cleanup ghc-flymake-get-real-file-name))
 
 (defconst ghc-flymake-err-line-patterns
   '("^\\(.*\\.l?hs\\):\\([0-9]+\\):\\([0-9]+\\):[ ]*\\(.+\\)" 1 2 3 4))
@@ -58,22 +56,38 @@
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(defun ghc-flymake-get-real-file-name (path)
+  (let ((bufnam (buffer-name))
+	(filnam (file-name-nondirectory path)))
+    (if (string= bufnam filnam)
+	bufnam
+      path)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
 (defun ghc-flymake-display-errors ()
   (interactive)
   (if (not (ghc-flymake-have-errs-p))
       (message "No errors or warnings")
-    (let ((buf (get-buffer-create ghc-error-buffer-name))
-	  (title (ghc-flymake-err-title))
+    (let ((title (ghc-flymake-err-title))
 	  (errs (ghc-flymake-err-list)))
-      (with-current-buffer buf
-	(erase-buffer)
-	(ghc-flymake-insert-errors title errs))
-      (display-buffer buf))))
+      (ghc-display
+       nil
+       (lambda (&rest ignore)
+	 (insert title "\n\n")
+	 (mapc (lambda (x) (insert x "\n")) errs))))))
 
-(defun ghc-flymake-insert-errors (title errs)
-  (save-excursion
-    (insert title "\n\n")
-    (mapc (lambda (x) (insert (ghc-replace-character x ghc-null ghc-newline) "\n")) errs)))
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(defun ghc-flymake-jump ()
+  (interactive)
+  (if (not (ghc-flymake-have-errs-p))
+      (message "No errors or warnings")
+    (let* ((acts (ghc-flymake-act-list))
+	   (act (car acts)))
+      (if (not act)
+	  (message "No destination")
+	(eval act)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -166,6 +180,9 @@
 (defun ghc-flymake-err-get-title (x) (nth 0 x))
 (defun ghc-flymake-err-get-errs (x) (nth 1 x))
 
+(defun ghc-flymake-err-get-err-msg (x) (nth 0 x))
+(defun ghc-flymake-err-get-err-act (x) (nth 1 x))
+
 (defalias 'ghc-flymake-have-errs-p 'ghc-flymake-data)
 
 (defun ghc-flymake-data ()
@@ -177,6 +194,9 @@
   (ghc-flymake-err-get-title (ghc-flymake-data)))
 
 (defun ghc-flymake-err-list ()
-  (mapcar 'car (ghc-flymake-err-get-errs (ghc-flymake-data))))
+  (mapcar 'ghc-flymake-err-get-err-msg (ghc-flymake-err-get-errs (ghc-flymake-data))))
+
+(defun ghc-flymake-act-list ()
+  (mapcar 'ghc-flymake-err-get-err-act (ghc-flymake-err-get-errs (ghc-flymake-data))))
 
 (provide 'ghc-flymake)
