@@ -17,7 +17,7 @@
 	 (expr (if ask (ghc-read-expression expr0) expr0))
 	 (file (buffer-file-name))
 	 (cmds (list "info" file modname expr)))
-    (ghc-display-information cmds)))
+    (ghc-display-information cmds nil)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -54,7 +54,7 @@
   (ghc-type-clear-overlay)
   (setq after-change-functions
 	(cons 'ghc-type-clear-overlay after-change-functions))
-  (set (make-local-variable 'post-command-hook) 'ghc-type-post-command-hook))
+  (add-hook 'post-command-hook 'ghc-type-post-command-hook))
 
 (defun ghc-type-clear-overlay (&optional beg end len)
   (when (overlayp ghc-type-overlay)
@@ -63,7 +63,8 @@
     (move-overlay ghc-type-overlay 0 0)))
 
 (defun ghc-type-post-command-hook ()
-  (when (and (overlayp ghc-type-overlay)
+  (when (and (eq major-mode 'haskell-mode)
+	     (overlayp ghc-type-overlay)
 	     (/= (ghc-type-get-point) (point)))
     (ghc-type-clear-overlay)))
 
@@ -124,31 +125,26 @@
   (interactive)
   (let* ((file (buffer-file-name))
 	 (cmds (list "expand" file)))
-    (ghc-display-information cmds)))
+    (ghc-display-information cmds t)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Display
 ;;;
 
-(defun ghc-display-information (cmds)
+(defun ghc-display-information (cmds fontify)
   (interactive)
   (if (not (ghc-which ghc-module-command))
       (message "%s not found" ghc-module-command)
-    (let ((cdir default-directory)
-	  (buf (get-buffer-create ghc-error-buffer-name)))
-      (with-current-buffer buf
-	(erase-buffer)
-	(insert
-	 (with-temp-buffer
-	   (cd cdir)
-	   (apply 'call-process ghc-module-command nil t nil
-		  (append (ghc-make-ghc-options) cmds))
-	   (buffer-substring (point-min) (1- (point-max)))))
-	(goto-char (point-min))
-	(haskell-font-lock-defaults-create)
-	(turn-on-haskell-font-lock))
-      (display-buffer buf))))
+    (ghc-display
+     fontify
+     (lambda (cdir)
+       (insert
+	(with-temp-buffer
+	  (cd cdir)
+	  (apply 'call-process ghc-module-command nil t nil
+		 (append (ghc-make-ghc-options) cmds))
+	  (buffer-substring (point-min) (1- (point-max)))))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
