@@ -16,7 +16,7 @@
 
 ;;; Code:
 
-(defconst ghc-version "1.10.10")
+(defconst ghc-version "1.10.15")
 
 ;; (eval-when-compile
 ;;  (require 'haskell-mode))
@@ -27,6 +27,7 @@
 (require 'ghc-flymake)
 (require 'ghc-command)
 (require 'ghc-ins-mod)
+(require 'ghc-indent)
 (require 'dabbrev)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -53,7 +54,10 @@
 (defvar ghc-toggle-key      "\C-c\C-c")
 (defvar ghc-module-key      "\C-c\C-m")
 (defvar ghc-expand-key      "\C-c\C-e")
+(defvar ghc-jump-key        "\C-c\C-j")
 (defvar ghc-hoogle-key      (format "\C-c%c" (ghc-find-C-h)))
+(defvar ghc-shallower-key   "\C-c<")
+(defvar ghc-deeper-key      "\C-c>")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -72,6 +76,7 @@
     (define-key haskell-mode-map ghc-type-key        'ghc-show-type)
     (define-key haskell-mode-map ghc-info-key        'ghc-show-info)
     (define-key haskell-mode-map ghc-expand-key      'ghc-expand-th)
+    (define-key haskell-mode-map ghc-jump-key        'ghc-flymake-jump)
     (define-key haskell-mode-map ghc-import-key      'ghc-import-module)
     (define-key haskell-mode-map ghc-previous-key    'flymake-goto-prev-error)
     (define-key haskell-mode-map ghc-next-key        'flymake-goto-next-error)
@@ -82,6 +87,8 @@
     (define-key haskell-mode-map ghc-toggle-key      'ghc-flymake-toggle-command)
     (define-key haskell-mode-map ghc-module-key      'ghc-insert-module)
     (define-key haskell-mode-map ghc-hoogle-key      'haskell-hoogle)
+    (define-key haskell-mode-map ghc-shallower-key   'ghc-make-indent-shallower)
+    (define-key haskell-mode-map ghc-deeper-key      'ghc-make-indent-deeper)
     (ghc-comp-init)
     (setq ghc-initialized t)))
 
